@@ -1,9 +1,9 @@
 """
-PyGoogleChart - A complete Python wrapper for the Google Chart API
+pygooglechart - A complete Python wrapper for the Google Chart API
 
 http://pygooglechart.slowchop.com/
 
-Copyright 2007 Gerald Kaszuba
+Copyright 2007-2009 Gerald Kaszuba
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 """
+from __future__ import division
 
 import os
 import urllib
@@ -26,21 +27,29 @@ import urllib2
 import math
 import random
 import re
+import warnings
+import copy
 
 # Helper variables and functions
 # -----------------------------------------------------------------------------
 
-__version__ = '0.2.0'
+__version__ = '0.3.0'
+__author__ = 'Gerald Kaszuba'
 
 reo_colour = re.compile('^([A-Fa-f0-9]{2,2}){3,4}$')
 
-
 def _check_colour(colour):
     if not reo_colour.match(colour):
         raise InvalidParametersException('Colours need to be in ' \
             'RRGGBB or RRGGBBAA format. One of your colours has %s' % \
             colour)
 
+
+def _reset_warnings():
+    """Helper function to reset all warnings. Used by the unit tests."""
+    globals()['__warningregistry__'] = None
+
+
 # Exception Classes
 # -----------------------------------------------------------------------------
 
@@ -69,6 +78,16 @@ class BadContentTypeException(PyGoogleChartException):
     pass
 
 
+class AbstractClassException(PyGoogleChartException):
+    pass
+
+
+class UnknownChartType(PyGoogleChartException):
+    pass
+
+class UnknownCountryCodeException(PyGoogleChartException):
+    pass
+
 # Data Classes
 # -----------------------------------------------------------------------------
 
@@ -76,44 +95,52 @@ class BadContentTypeException(PyGoogleChartException):
 class Data(object):
 
     def __init__(self, data):
-        assert(type(self) != Data)  # This is an abstract class
+        if type(self) == Data:
+            raise AbstractClassException('This is an abstract class')
         self.data = data
 
     @classmethod
     def float_scale_value(cls, value, range):
         lower, upper = range
-        max_value = cls.max_value()
-        scaled = (value-lower) * (float(max_value)/(upper-lower))
+        assert(upper > lower)
+        scaled = (value - lower) * (cls.max_value / (upper - lower))
         return scaled
 
     @classmethod
     def clip_value(cls, value):
-        clipped = max(0, min(value, cls.max_value()))
-        return clipped
+        return max(0, min(value, cls.max_value))
 
     @classmethod
     def int_scale_value(cls, value, range):
-        scaled = int(round(cls.float_scale_value(value, range)))
-        return scaled
+        return int(round(cls.float_scale_value(value, range)))
 
     @classmethod
     def scale_value(cls, value, range):
         scaled = cls.int_scale_value(value, range)
         clipped = cls.clip_value(scaled)
+        Data.check_clip(scaled, clipped)
         return clipped
 
+    @staticmethod
+    def check_clip(scaled, clipped):
+        if clipped != scaled:
+            warnings.warn('One or more of of your data points has been '
+                'clipped because it is out of range.')
+
+
 class SimpleData(Data):
+
+    max_value = 61
     enc_map = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789'
 
     def __repr__(self):
-        max_value = self.max_value()
         encoded_data = []
         for data in self.data:
             sub_data = []
             for value in data:
                 if value is None:
                     sub_data.append('_')
-                elif value >= 0 and value <= max_value:
+                elif value >= 0 and value <= self.max_value:
                     sub_data.append(SimpleData.enc_map[value])
                 else:
                     raise DataOutOfRangeException('cannot encode value: %d'
@@ -121,56 +148,42 @@ class SimpleData(Data):
             encoded_data.append(''.join(sub_data))
         return 'chd=s:' + ','.join(encoded_data)
 
-    @staticmethod
-    def max_value():
-        return 61
 
 class TextData(Data):
 
+    max_value = 100
+
     def __repr__(self):
-        max_value = self.max_value()
         encoded_data = []
         for data in self.data:
             sub_data = []
             for value in data:
                 if value is None:
                     sub_data.append(-1)
-                elif value >= 0 and value <= max_value:
+                elif value >= 0 and value <= self.max_value:
                     sub_data.append("%.1f" % float(value))
                 else:
                     raise DataOutOfRangeException()
             encoded_data.append(','.join(sub_data))
-        return 'chd=t:' + '|'.join(encoded_data)
-
-    @staticmethod
-    def max_value():
-        return 100
-
-    @classmethod
-    def scale_value(cls, value, range):
-        lower, upper = range
-        if upper > lower:
-            max_value = cls.max_value()
-            scaled = (float(value) - lower) * max_value / upper
-            clipped = max(0, min(scaled, max_value))
-            return clipped
-        else:
-            return lower
+        return 'chd=t:' + '%7c'.join(encoded_data)
 
     @classmethod
     def scale_value(cls, value, range):
         # use float values instead of integers because we don't need an encode
         # map index
-        scaled = cls.float_scale_value(value,range)
+        scaled = cls.float_scale_value(value, range)
         clipped = cls.clip_value(scaled)
+        Data.check_clip(scaled, clipped)
         return clipped
 
+
 class ExtendedData(Data):
+
+    max_value = 4095
     enc_map = \
         'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789-.'
 
     def __repr__(self):
-        max_value = self.max_value()
         encoded_data = []
         enc_size = len(ExtendedData.enc_map)
         for data in self.data:
@@ -178,7 +191,7 @@ class ExtendedData(Data):
             for value in data:
                 if value is None:
                     sub_data.append('__')
-                elif value >= 0 and value <= max_value:
+                elif value >= 0 and value <= self.max_value:
                     first, second = divmod(int(value), enc_size)
                     sub_data.append('%s%s' % (
                         ExtendedData.enc_map[first],
@@ -190,16 +203,13 @@ class ExtendedData(Data):
             encoded_data.append(''.join(sub_data))
         return 'chd=e:' + ','.join(encoded_data)
 
-    @staticmethod
-    def max_value():
-        return 4095
-
 
 # Axis Classes
 # -----------------------------------------------------------------------------
 
 
 class Axis(object):
+
     BOTTOM = 'x'
     TOP = 't'
     LEFT = 'y'
@@ -250,7 +260,7 @@ class LabelAxis(Axis):
         self.values = [str(a) for a in values]
 
     def __repr__(self):
-        return '%i:|%s' % (self.axis_index, '|'.join(self.values))
+        return '%i:%%7c%s' % (self.axis_index, '%7c'.join(self.values))
 
 
 class RangeAxis(Axis):
@@ -284,21 +294,26 @@ class Chart(object):
     LINEAR_STRIPES = 'ls'
 
     def __init__(self, width, height, title=None, legend=None, colours=None,
-                 auto_scale=True, x_range=None, y_range=None):
-        assert(type(self) != Chart)  # This is an abstract class
+            auto_scale=True, x_range=None, y_range=None,
+            colours_within_series=None):
+        if type(self) == Chart:
+            raise AbstractClassException('This is an abstract class')
         assert(isinstance(width, int))
         assert(isinstance(height, int))
         self.width = width
         self.height = height
         self.data = []
         self.set_title(title)
+        self.set_title_style(None, None)
         self.set_legend(legend)
+        self.set_legend_position(None)
         self.set_colours(colours)
+        self.set_colours_within_series(colours_within_series)
 
         # Data for scaling.
-        self.auto_scale = auto_scale    # Whether to automatically scale data
-        self.x_range = x_range          # (min, max) x-axis range for scaling
-        self.y_range = y_range          # (min, max) y-axis range for scaling
+        self.auto_scale = auto_scale  # Whether to automatically scale data
+        self.x_range = x_range  # (min, max) x-axis range for scaling
+        self.y_range = y_range  # (min, max) y-axis range for scaling
         self.scaled_data_class = None
         self.scaled_x_range = None
         self.scaled_y_range = None
@@ -317,6 +332,8 @@ class Chart(object):
         self.markers = []
         self.line_styles = {}
         self.grid = None
+        self.title_colour = None
+        self.title_font_size = None
 
     # URL generation
     # -------------------------------------------------------------------------
@@ -334,18 +351,25 @@ class Chart(object):
         # optional arguments
         if self.title:
             url_bits.append('chtt=%s' % self.title)
+        if self.title_colour and self.title_font_size:
+            url_bits.append('chts=%s,%s' % (self.title_colour, \
+                self.title_font_size))
         if self.legend:
-            url_bits.append('chdl=%s' % '|'.join(self.legend))
+            url_bits.append('chdl=%s' % '%7c'.join(self.legend))
+        if self.legend_position:
+            url_bits.append('chdlp=%s' % (self.legend_position))
         if self.colours:
-            url_bits.append('chco=%s' % ','.join(self.colours))
+            url_bits.append('chco=%s' % ','.join(self.colours))            
+        if self.colours_within_series:
+            url_bits.append('chco=%s' % '%7c'.join(self.colours_within_series))
         ret = self.fill_to_url()
         if ret:
             url_bits.append(ret)
         ret = self.axis_to_url()
         if ret:
-            url_bits.append(ret)
+            url_bits.append(ret)                    
         if self.markers:
-            url_bits.append(self.markers_to_url())
+            url_bits.append(self.markers_to_url())        
         if self.line_styles:
             style = []
             for index in xrange(max(self.line_styles) + 1):
@@ -354,7 +378,7 @@ class Chart(object):
                 else:
                     values = ('1', )
                 style.append(','.join(values))
-            url_bits.append('chls=%s' % '|'.join(style))
+            url_bits.append('chls=%s' % '%7c'.join(style))
         if self.grid:
             url_bits.append('chg=%s' % self.grid)
         return url_bits
@@ -369,7 +393,7 @@ class Chart(object):
             raise BadContentTypeException('Server responded with a ' \
                 'content-type of %s' % opener.headers['content-type'])
 
-        open(file_name, 'wb').write(urllib.urlopen(self.get_url()).read())
+        open(file_name, 'wb').write(opener.read())
 
     # Simple settings
     # -------------------------------------------------------------------------
@@ -380,6 +404,14 @@ class Chart(object):
         else:
             self.title = None
 
+    def set_title_style(self, colour=None, font_size=None):
+        if not colour is None:
+            _check_colour(colour)
+        if not colour and not font_size:
+            return
+        self.title_colour = colour or '333333'
+        self.title_font_size = font_size or 13.5
+
     def set_legend(self, legend):
         """legend needs to be a list, tuple or None"""
         assert(isinstance(legend, list) or isinstance(legend, tuple) or
@@ -389,6 +421,12 @@ class Chart(object):
         else:
             self.legend = None
 
+    def set_legend_position(self, legend_position):
+        if legend_position:
+            self.legend_position = urllib.quote(legend_position)
+        else:    
+            self.legend_position = None
+
     # Chart colours
     # -------------------------------------------------------------------------
 
@@ -402,6 +440,16 @@ class Chart(object):
                 _check_colour(col)
         self.colours = colours
 
+    def set_colours_within_series(self, colours):
+        # colours needs to be a list, tuple or None
+        assert(isinstance(colours, list) or isinstance(colours, tuple) or
+            colours is None)
+        # make sure the colours are in the right format
+        if colours:
+            for col in colours:
+                _check_colour(col)
+        self.colours_within_series = colours        
+
     # Background/Chart colours
     # -------------------------------------------------------------------------
 
@@ -416,7 +464,7 @@ class Chart(object):
         assert(angle >= 0 and angle <= 90)
         assert(len(args) % 2 == 0)
         args = list(args)  # args is probably a tuple and we need to mutate
-        for a in xrange(len(args) / 2):
+        for a in xrange(int(len(args) / 2)):
             col = args[a * 2]
             offset = args[a * 2 + 1]
             _check_colour(col)
@@ -443,7 +491,7 @@ class Chart(object):
                 areas.append('%s,%s,%s' % (area, self.fill_types[area], \
                     self.fill_area[area]))
         if areas:
-            return 'chf=' + '|'.join(areas)
+            return 'chf=' + '%7c'.join(areas)
 
     # Data
     # -------------------------------------------------------------------------
@@ -467,14 +515,19 @@ class Chart(object):
         else:
             return ExtendedData
 
+    def _filter_none(self, data):
+        return [r for r in data if r is not None]
+
     def data_x_range(self):
         """Return a 2-tuple giving the minimum and maximum x-axis
         data range.
         """
         try:
-            lower = min([min(s) for type, s in self.annotated_data()
+            lower = min([min(self._filter_none(s))
+                         for type, s in self.annotated_data()
                          if type == 'x'])
-            upper = max([max(s) for type, s in self.annotated_data()
+            upper = max([max(self._filter_none(s))
+                         for type, s in self.annotated_data()
                          if type == 'x'])
             return (lower, upper)
         except ValueError:
@@ -485,9 +538,11 @@ class Chart(object):
         data range.
         """
         try:
-            lower = min([min(s) for type, s in self.annotated_data()
+            lower = min([min(self._filter_none(s))
+                         for type, s in self.annotated_data()
                          if type == 'y'])
-            upper = max([max(s) for type, s in self.annotated_data()
+            upper = max([max(self._filter_none(s)) + 1
+                         for type, s in self.annotated_data()
                          if type == 'y'])
             return (lower, upper)
         except ValueError:
@@ -513,14 +568,14 @@ class Chart(object):
         if x_range is None:
             x_range = self.data_x_range()
             if x_range and x_range[0] > 0:
-                x_range = (0, x_range[1])
+                x_range = (x_range[0], x_range[1])
         self.scaled_x_range = x_range
 
         # Determine the y-axis range for scaling.
         if y_range is None:
             y_range = self.data_y_range()
             if y_range and y_range[0] > 0:
-                y_range = (0, y_range[1])
+                y_range = (y_range[0], y_range[1])
         self.scaled_y_range = y_range
 
         scaled_data = []
@@ -531,8 +586,14 @@ class Chart(object):
                 scale_range = y_range
             elif type == 'marker-size':
                 scale_range = (0, max(dataset))
-            scaled_data.append([data_class.scale_value(v, scale_range)
-                                for v in dataset])
+            scaled_dataset = []
+            for v in dataset:
+                if v is None:
+                    scaled_dataset.append(None)
+                else:
+                    scaled_dataset.append(
+                        data_class.scale_value(v, scale_range))
+            scaled_data.append(scaled_dataset)
         return scaled_data
 
     def add_data(self, data):
@@ -559,7 +620,7 @@ class Chart(object):
 
     def set_axis_labels(self, axis_type, values):
         assert(axis_type in Axis.TYPES)
-        values = [ urllib.quote(a) for a in values ]
+        values = [urllib.quote(str(a)) for a in values]
         axis_index = len(self.axis)
         axis = LabelAxis(axis_index, axis_type, values)
         self.axis.append(axis)
@@ -609,30 +670,39 @@ class Chart(object):
         url_bits = []
         url_bits.append('chxt=%s' % ','.join(available_axis))
         if label_axis:
-            url_bits.append('chxl=%s' % '|'.join(label_axis))
+            url_bits.append('chxl=%s' % '%7c'.join(label_axis))
         if range_axis:
-            url_bits.append('chxr=%s' % '|'.join(range_axis))
+            url_bits.append('chxr=%s' % '%7c'.join(range_axis))
         if positions:
-            url_bits.append('chxp=%s' % '|'.join(positions))
+            url_bits.append('chxp=%s' % '%7c'.join(positions))
         if styles:
-            url_bits.append('chxs=%s' % '|'.join(styles))
+            url_bits.append('chxs=%s' % '%7c'.join(styles))
         return '&'.join(url_bits)
 
     # Markers, Ranges and Fill area (chm)
     # -------------------------------------------------------------------------
 
-    def markers_to_url(self):
-        return 'chm=%s' % '|'.join([','.join(a) for a in self.markers])
+    def markers_to_url(self):        
+        return 'chm=%s' % '%7c'.join([','.join(a) for a in self.markers])
 
     def add_marker(self, index, point, marker_type, colour, size, priority=0):
         self.markers.append((marker_type, colour, str(index), str(point), \
             str(size), str(priority)))
 
     def add_horizontal_range(self, colour, start, stop):
-        self.markers.append(('r', colour, '1', str(start), str(stop)))
+        self.markers.append(('r', colour, '0', str(start), str(stop)))
+
+    def add_data_line(self, colour, data_set, size, priority=0):
+        self.markers.append(('D', colour, str(data_set), '0', str(size), \
+            str(priority)))
+
+    def add_marker_text(self, string, colour, data_set, data_point, size, \
+            priority=0):
+        self.markers.append((str(string), colour, str(data_set), \
+            str(data_point), str(size), str(priority)))        
 
     def add_vertical_range(self, colour, start, stop):
-        self.markers.append(('R', colour, '1', str(start), str(stop)))
+        self.markers.append(('R', colour, '0', str(start), str(stop)))
 
     def add_fill_range(self, colour, index_start, index_end):
         self.markers.append(('b', colour, str(index_start), str(index_end), \
@@ -675,16 +745,14 @@ class ScatterChart(Chart):
             # markers.
             yield ('marker-size', self.data[2])
 
+
 class LineChart(Chart):
 
     def __init__(self, *args, **kwargs):
-        assert(type(self) != LineChart)  # This is an abstract class
+        if type(self) == LineChart:
+            raise AbstractClassException('This is an abstract class')
         Chart.__init__(self, *args, **kwargs)
 
-#    def get_url_bits(self, data_class=None):
-#        url_bits = Chart.get_url_bits(self, data_class=data_class)
-#        return url_bits
-
 
 class SimpleLineChart(LineChart):
 
@@ -696,11 +764,13 @@ class SimpleLineChart(LineChart):
         for dataset in self.data:
             yield ('y', dataset)
 
+
 class SparkLineChart(SimpleLineChart):
 
     def type_to_url(self):
         return 'cht=ls'
 
+
 class XYLineChart(LineChart):
 
     def type_to_url(self):
@@ -714,10 +784,12 @@ class XYLineChart(LineChart):
             else:
                 yield ('y', dataset)
 
+
 class BarChart(Chart):
 
     def __init__(self, *args, **kwargs):
-        assert(type(self) != BarChart)  # This is an abstract class
+        if type(self) == BarChart:
+            raise AbstractClassException('This is an abstract class')
         Chart.__init__(self, *args, **kwargs)
         self.bar_width = None
         self.zero_lines = {}
@@ -762,7 +834,8 @@ class StackedVerticalBarChart(BarChart):
 class GroupedBarChart(BarChart):
 
     def __init__(self, *args, **kwargs):
-        assert(type(self) != GroupedBarChart)  # This is an abstract class
+        if type(self) == GroupedBarChart:
+            raise AbstractClassException('This is an abstract class')
         BarChart.__init__(self, *args, **kwargs)
         self.bar_spacing = None
         self.group_spacing = None
@@ -782,8 +855,8 @@ class GroupedBarChart(BarChart):
             skip_chbh=True)
         if self.group_spacing is not None:
             if self.bar_spacing is None:
-                raise InvalidParametersException('Bar spacing is required to ' \
-                    'be set when setting group spacing')
+                raise InvalidParametersException('Bar spacing is required ' \
+                    'to be set when setting group spacing')
             if self.bar_width is None:
                 raise InvalidParametersException('Bar width is required to ' \
                     'be set when setting bar spacing')
@@ -818,9 +891,13 @@ class GroupedVerticalBarChart(GroupedBarChart):
 class PieChart(Chart):
 
     def __init__(self, *args, **kwargs):
-        assert(type(self) != PieChart)  # This is an abstract class
+        if type(self) == PieChart:
+            raise AbstractClassException('This is an abstract class')
         Chart.__init__(self, *args, **kwargs)
         self.pie_labels = []
+        if self.y_range:
+            warnings.warn('y_range is not used with %s.' % \
+                (self.__class__.__name__))
 
     def set_pie_labels(self, labels):
         self.pie_labels = [urllib.quote(a) for a in labels]
@@ -828,14 +905,19 @@ class PieChart(Chart):
     def get_url_bits(self, data_class=None):
         url_bits = Chart.get_url_bits(self, data_class=data_class)
         if self.pie_labels:
-            url_bits.append('chl=%s' % '|'.join(self.pie_labels))
+            url_bits.append('chl=%s' % '%7c'.join(self.pie_labels))
         return url_bits
 
     def annotated_data(self):
         # Datasets are all y-axis data. However, there should only be
         # one dataset for pie charts.
         for dataset in self.data:
-            yield ('y', dataset)
+            yield ('x', dataset)
+
+    def scaled_data(self, data_class, x_range=None, y_range=None):
+        if not x_range:
+            x_range = [0, sum(self.data[0])]
+        return Chart.scaled_data(self, data_class, x_range, self.y_range)
 
 
 class PieChart2D(PieChart):
@@ -865,6 +947,7 @@ class RadarChart(Chart):
     def type_to_url(self):
         return 'cht=r'
 
+
 class SplineRadarChart(RadarChart):
 
     def type_to_url(self):
@@ -877,12 +960,72 @@ class MapChart(Chart):
         Chart.__init__(self, *args, **kwargs)
         self.geo_area = 'world'
         self.codes = []
-
+        self.__areas = ('africa', 'asia', 'europe', 'middle_east',
+            'south_america', 'usa', 'world')
+        self.__ccodes = (
+            'AD', 'AE', 'AF', 'AG', 'AI', 'AL', 'AM', 'AN', 'AO', 'AQ', 'AR',
+            'AS', 'AT', 'AU', 'AW', 'AX', 'AZ', 'BA', 'BB', 'BD', 'BE', 'BF',
+            'BG', 'BH', 'BI', 'BJ', 'BL', 'BM', 'BN', 'BO', 'BR', 'BS', 'BT',
+            'BV', 'BW', 'BY', 'BZ', 'CA', 'CC', 'CD', 'CF', 'CG', 'CH', 'CI',
+            'CK', 'CL', 'CM', 'CN', 'CO', 'CR', 'CU', 'CV', 'CX', 'CY', 'CZ',
+            'DE', 'DJ', 'DK', 'DM', 'DO', 'DZ', 'EC', 'EE', 'EG', 'EH', 'ER',
+            'ES', 'ET', 'FI', 'FJ', 'FK', 'FM', 'FO', 'FR', 'GA', 'GB', 'GD',
+            'GE', 'GF', 'GG', 'GH', 'GI', 'GL', 'GM', 'GN', 'GP', 'GQ', 'GR',
+            'GS', 'GT', 'GU', 'GW', 'GY', 'HK', 'HM', 'HN', 'HR', 'HT', 'HU',
+            'ID', 'IE', 'IL', 'IM', 'IN', 'IO', 'IQ', 'IR', 'IS', 'IT', 'JE',
+            'JM', 'JO', 'JP', 'KE', 'KG', 'KH', 'KI', 'KM', 'KN', 'KP', 'KR',
+            'KW', 'KY', 'KZ', 'LA', 'LB', 'LC', 'LI', 'LK', 'LR', 'LS', 'LT',
+            'LU', 'LV', 'LY', 'MA', 'MC', 'MD', 'ME', 'MF', 'MG', 'MH', 'MK',
+            'ML', 'MM', 'MN', 'MO', 'MP', 'MQ', 'MR', 'MS', 'MT', 'MU', 'MV',
+            'MW', 'MX', 'MY', 'MZ', 'NA', 'NC', 'NE', 'NF', 'NG', 'NI', 'NL',
+            'NO', 'NP', 'NR', 'NU', 'NZ', 'OM', 'PA', 'PE', 'PF', 'PG', 'PH',
+            'PK', 'PL', 'PM', 'PN', 'PR', 'PS', 'PT', 'PW', 'PY', 'QA', 'RE',
+            'RO', 'RS', 'RU', 'RW', 'SA', 'SB', 'SC', 'SD', 'SE', 'SG', 'SH',
+            'SI', 'SJ', 'SK', 'SL', 'SM', 'SN', 'SO', 'SR', 'ST', 'SV', 'SY',
+            'SZ', 'TC', 'TD', 'TF', 'TG', 'TH', 'TJ', 'TK', 'TL', 'TM', 'TN',
+            'TO', 'TR', 'TT', 'TV', 'TW', 'TZ', 'UA', 'UG', 'UM', 'US', 'UY',
+            'UZ', 'VA', 'VC', 'VE', 'VG', 'VI', 'VN', 'VU', 'WF', 'WS', 'YE',
+            'YT', 'ZA', 'ZM', 'ZW')
+        
     def type_to_url(self):
         return 'cht=t'
 
     def set_codes(self, codes):
-        self.codes = codes
+        '''Set the country code map for the data.
+        Codes given in a list.
+
+        i.e. DE - Germany
+             AT - Austria
+             US - United States
+        '''
+
+        codemap = ''
+        
+        for cc in codes:
+            cc = cc.upper()
+            if cc in self.__ccodes:
+                codemap += cc
+            else:
+                raise UnknownCountryCodeException(cc)
+            
+        self.codes = codemap
+
+    def set_geo_area(self, area):
+        '''Sets the geo area for the map.
+
+        * africa
+        * asia
+        * europe
+        * middle_east
+        * south_america
+        * usa
+        * world
+        '''
+        
+        if area in self.__areas:
+            self.geo_area = area
+        else:
+            raise UnknownChartType('Unknown chart type for maps: %s' %area)
 
     def get_url_bits(self, data_class=None):
         url_bits = Chart.get_url_bits(self, data_class=data_class)
@@ -891,76 +1034,122 @@ class MapChart(Chart):
             url_bits.append('chld=%s' % ''.join(self.codes))
         return url_bits
 
+    def add_data_dict(self, datadict):
+        '''Sets the data and country codes via a dictionary.
+
+        i.e. {'DE': 50, 'GB': 30, 'AT': 70}
+        '''
+
+        self.set_codes(datadict.keys())
+        self.add_data(datadict.values())
+
 
 class GoogleOMeterChart(PieChart):
     """Inheriting from PieChart because of similar labeling"""
 
+    def __init__(self, *args, **kwargs):
+        PieChart.__init__(self, *args, **kwargs)
+        if self.auto_scale and not self.x_range:
+            warnings.warn('Please specify an x_range with GoogleOMeterChart, '
+                'otherwise one arrow will always be at the max.')
+
     def type_to_url(self):
         return 'cht=gom'
 
 
-def test():
-    chart = PieChart2D(320, 200)
-    chart = ScatterChart(320, 200)
-    chart = SimpleLineChart(320, 200)
-    chart = GroupedVerticalBarChart(320, 200)
-#    chart = SplineRadarChart(500, 500)
-#    chart = MapChart(440, 220)
-#    chart = GoogleOMeterChart(440, 220, x_range=(0, 100))
-    sine_data = [math.sin(float(a) / math.pi) * 100 for a in xrange(100)]
-    random_data = [random.random() * 100 for a in xrange(100)]
-    random_data2 = [random.random() * 50 for a in xrange(100)]
-#    chart.set_bar_width(50)
-#    chart.set_bar_spacing(0)
-    chart.add_data(sine_data)
-    chart.add_data(random_data)
-    chart.set_zero_line(1, .5)
-#    chart.add_data(random_data2)
-#    chart.set_line_style(0, thickness=5)
-#    chart.set_line_style(1, thickness=2, line_segment=10, blank_segment=5)
-#    chart.set_title('heloooo weeee')
-#    chart.set_legend(('sine wave', 'random * x'))
-    chart.set_colours(('ee2000', 'DDDDAA', 'fF03f2'))
-#    chart.fill_solid(Chart.ALPHA, '123456')
-#    chart.fill_linear_gradient(Chart.ALPHA, 20, '004070', 1, '300040', 0,
-#        'aabbcc55', 0.5)
-#    chart.fill_linear_stripes(Chart.CHART, 20, '204070', .2, '300040', .2,
-#        'aabbcc00', 0.2)
-#    axis_left_index = chart.set_axis_range(Axis.LEFT, 0, 10)
-#    axis_right_index = chart.set_axis_range(Axis.RIGHT, 5, 30)
-#    axis_bottom_index = chart.set_axis_labels(Axis.BOTTOM, [1, 25, 95])
-#    chart.set_axis_positions(axis_bottom_index, [1, 25, 95])
-#    chart.set_axis_style(axis_bottom_index, '003050', 15)
+class QRChart(Chart):
 
-#    chart.set_pie_labels(('apples', 'oranges', 'bananas'))
+    def __init__(self, *args, **kwargs):
+        Chart.__init__(self, *args, **kwargs)
+        self.encoding = None
+        self.ec_level = None
+        self.margin = None
 
-#    chart.set_grid(10, 10)
-#    for a in xrange(0, 100, 10):
-#        chart.add_marker(1, a, 'a', 'AACA20', 10)
+    def type_to_url(self):
+        return 'cht=qr'
 
-#    chart.add_horizontal_range('00A020', .2, .5)
-#    chart.add_vertical_range('00c030', .2, .4)
+    def data_to_url(self, data_class=None):
+        if not self.data:
+            raise NoDataGivenException()
+        return 'chl=%s' % urllib.quote(self.data[0])
 
-#    chart.add_fill_simple('303030A0')
+    def get_url_bits(self, data_class=None):
+        url_bits = Chart.get_url_bits(self, data_class=data_class)
+        if self.encoding:
+            url_bits.append('choe=%s' % self.encoding)
+        if self.ec_level:
+            url_bits.append('chld=%s%%7c%s' % (self.ec_level, self.margin))
+        return url_bits
 
-#    chart.set_codes(['AU', 'AT', 'US'])
-#    chart.add_data([1,2,3])
-#    chart.set_colours(('EEEEEE', '000000', '00FF00'))
+    def set_encoding(self, encoding):
+        self.encoding = encoding
 
-#    chart.add_data([50,75])
-#    chart.set_pie_labels(('apples', 'oranges'))
-
-    url = chart.get_url()
-    print url
-
-    chart.download('test.png')
-
-    if 1:
-        data = urllib.urlopen(chart.get_url()).read()
-        open('meh.png', 'wb').write(data)
-        os.system('eog meh.png')
+    def set_ec(self, level, margin):
+        self.ec_level = level
+        self.margin = margin
 
 
-if __name__ == '__main__':
-    test()
+class ChartGrammar(object):
+
+    def __init__(self):
+        self.grammar = None
+        self.chart = None
+
+    def parse(self, grammar):
+        self.grammar = grammar
+        self.chart = self.create_chart_instance()
+
+        for attr in self.grammar:
+            if attr in ('w', 'h', 'type', 'auto_scale', 'x_range', 'y_range'):
+                continue  # These are already parsed in create_chart_instance
+            attr_func = 'parse_' + attr
+            if not hasattr(self, attr_func):
+                warnings.warn('No parser for grammar attribute "%s"' % (attr))
+                continue
+            getattr(self, attr_func)(grammar[attr])
+
+        return self.chart
+
+    def parse_data(self, data):
+        self.chart.data = data
+
+    @staticmethod
+    def get_possible_chart_types():
+        possible_charts = []
+        for cls_name in globals().keys():
+            if not cls_name.endswith('Chart'):
+                continue
+            cls = globals()[cls_name]
+            # Check if it is an abstract class
+            try:
+                a = cls(1, 1, auto_scale=False)
+                del a
+            except AbstractClassException:
+                continue
+            # Strip off "Class"
+            possible_charts.append(cls_name[:-5])
+        return possible_charts
+
+    def create_chart_instance(self, grammar=None):
+        if not grammar:
+            grammar = self.grammar
+        assert(isinstance(grammar, dict))  # grammar must be a dict
+        assert('w' in grammar)  # width is required
+        assert('h' in grammar)  # height is required
+        assert('type' in grammar)  # type is required
+        chart_type = grammar['type']
+        w = grammar['w']
+        h = grammar['h']
+        auto_scale = grammar.get('auto_scale', None)
+        x_range = grammar.get('x_range', None)
+        y_range = grammar.get('y_range', None)
+        types = ChartGrammar.get_possible_chart_types()
+        if chart_type not in types:
+            raise UnknownChartType('%s is an unknown chart type. Possible '
+                'chart types are %s' % (chart_type, ','.join(types)))
+        return globals()[chart_type + 'Chart'](w, h, auto_scale=auto_scale,
+            x_range=x_range, y_range=y_range)
+
+    def download(self):
+        pass
 
